@@ -11,16 +11,18 @@ from pydantic import (
     confloat,
     conint,
     conlist,
+    conset,
     constr,
     Field,
 )
 
+
 class Model(BaseModel):
     short_bytes: conbytes(min_length=2, max_length=10)
     strip_bytes: conbytes(strip_whitespace=True)
 
     short_str: constr(min_length=2, max_length=10)
-    regex_str: constr(regex='apple (pie|tart|sandwich)')
+    regex_str: constr(regex=r'^apple (pie|tart|sandwich)$')
     strip_str: constr(strip_whitespace=True)
 
     big_int: conint(gt=1000, lt=1024)
@@ -35,6 +37,7 @@ class Model(BaseModel):
     neg_float: NegativeFloat
 
     short_list: conlist(int, min_items=1, max_items=4)
+    short_set: conset(int, min_items=1, max_items=4)
 
     decimal_positive: condecimal(gt=0)
     decimal_negative: condecimal(lt=0)
