@@ -1,10 +1,12 @@
-from typing_extensions import Literal
+from typing import Literal
 
 from pydantic import BaseModel, ValidationError
 
+
 class Pie(BaseModel):
     flavor: Literal['apple', 'pumpkin']
 
+
 Pie(flavor='apple')
 Pie(flavor='pumpkin')
 try:
