@@ -1,18 +1,20 @@
-from pydantic import (
-    BaseModel, confloat,  StrictBool, StrictInt, ValidationError
-)
+from pydantic import BaseModel, StrictBool, StrictInt, ValidationError, confloat
+
 
 class StrictIntModel(BaseModel):
     strict_int: StrictInt
 
+
 try:
     StrictIntModel(strict_int=3.14159)
 except ValidationError as e:
     print(e)
 
+
 class ConstrainedFloatModel(BaseModel):
     constrained_float: confloat(strict=True, ge=0.0)
 
+
 try:
     ConstrainedFloatModel(constrained_float=3)
 except ValidationError as e:
@@ -23,9 +25,11 @@ try:
 except ValidationError as e:
     print(e)
 
+
 class StrictBoolModel(BaseModel):
     strict_bool: StrictBool
 
+
 try:
     StrictBoolModel(strict_bool='False')
 except ValidationError as e:
